import logging
from collections import namedtuple

import zope.interface
from certbot import errors, interfaces
from certbot.plugins import dns_common
from dnspod_sdk import DnspodClient

logger = logging.getLogger(__name__)

DomainInfo = namedtuple("DomainInfo", ["id", "name"])


@zope.interface.implementer(interfaces.IAuthenticator)
@zope.interface.provider(interfaces.IPluginFactory)
class Authenticator(dns_common.DNSAuthenticator):
    """DNS Authenticator for DNSPod

    This Authenticator uses the DNSPod API to fulfill a dns-01 challenge.
    """

    description = "使用DNS TXT记录获取证书（如果使用DNSPod管理DNS）"

    def __init__(self, *args, **kwargs):
        super(Authenticator, self).__init__(*args, **kwargs)
        self.credentials = None

    @classmethod
    def add_parser_arguments(cls, add):  # pylint: disable=arguments-differ
        super(Authenticator, cls).add_parser_arguments(add)
        add("credentials", help="DNSPod credentials INI file.")

    def more_info(self):  # pylint: disable=missing-function-docstring
        return "This plugin configures a DNS TXT record to respond to a dnspod " "challenge using the DNSPod API."

    def _setup_credentials(self):
        self.credentials = self._configure_credentials(
            "credentials",
            "DNSPod credentials INI file",
            {"token_id": "API token_id for DNSPod account", "token": "API token for DNSPod account"},
        )

    def _perform(self, domain, validation_name, validation):
        self._get_dnspod_client().add_txt_record(domain, validation_name, validation)

    def _cleanup(self, domain, validation_name, validation):
        self._get_dnspod_client().del_txt_record(domain, validation_name, validation)

    def _get_dnspod_client(self):
        return _DNSPodClient(self.credentials.conf("token_id"), self.credentials.conf("token"))


class _DNSPodClient(object):
    """
    Encapsulates all communication with the DNSPod API.
    """

    def __init__(self, token_id, token):
        user_agent = "Certbot DNS Plugins/1.0(me@codeif.com))"
        self.dc = DnspodClient(token_id, token, user_agent)

    def add_txt_record(self, domain_name, record_name, record_content):
        """
        Add a TXT record using the supplied information.

        :param str domain_name: The domain to use to associate the record with.
        :param str record_name: The record name (typically beginning with '_acme-challenge.').
        :param str record_content: The record content (typically the challenge validation).
        :raises certbot.errors.PluginError: if an error occurs communicating with the DNSPod API
        """
        domain_info = self._find_domain_info(domain_name)
        data = {
            "domain_id": domain_info.id,
            "sub_domain": self.get_sub_domain(record_name, domain_info),
            "record_type": "TXT",
            "value": record_content,
            "record_line": "默认",
        }
        j = self.dc.post("/Record.Create", data=data).json()
        self._check_response(j)

    def del_txt_record(self, domain_name, record_name, record_content):
        """
        Delete a TXT record using the supplied information.

        Note that both the record's name and content are used to ensure that similar records
        created concurrently (e.g., due to concurrent invocations of this plugin) are not deleted.

        Failures are logged, but not raised.

        :param str domain_name: The domain to use to associate the record with.
        :param str record_name: The record name (typically beginning with '_acme-challenge.').
        :param str record_content: The record content (typically the challenge validation).
        """
        domain_info = self._find_domain_info(domain_name)
        record_sub_domain = self.get_sub_domain(record_name, domain_info)
        record_ids = self._find_record_ids(domain_info.id, record_sub_domain)
        for record_id in record_ids:
            data = {
                "domain_id": domain_info.id,
                "record_id": record_id,
            }
            j = self.dc.post("/Record.Remove", data=data).json()
            self._check_response(j)

    def _check_response(self, j):
        status = j["status"]
        code = int(status["code"])
        if code != 1:
            raise errors.PluginError(f'DNSPod API response error, code-{code}, message-{status["message"]}')

    def get_sub_domain(self, record_name, domain_info):
        return record_name[0 : -len(domain_info.name) - 1]

    def _find_domain_info(self, domain_name):
        """
        Find the domain object for a given domain name.

        :param str domain_name: The domain name for which to find the corresponding Domain.
        :returns: The Domain, if found.
        :rtype: `~dnspod.DomainInfo`
        :raises certbot.errors.PluginError: if no matching Domain is found.
        """
        j = self.dc.post("/Domain.List").json()
        self._check_response(j)
        for domain_obj in j["domains"]:
            if domain_name.endswith(domain_obj["name"]):
                return DomainInfo(domain_obj["id"], domain_obj["name"])

    def _find_record_ids(self, domain_id, sub_domain):
        data = {
            "domain_id": domain_id,
            "sub_domain": sub_domain,
            "record_type": "TXT",
        }
        j = self.dc.post("/Record.List", data=data).json()
        code = int(j["status"]["code"])
        if code == 10:
            # 'code': '10', 'message': '记录列表为空'
            return []
        self._check_response(j)
        return [x["id"] for x in j["records"]]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['certbot_dnspod']

package_data = \
{'': ['*']}

install_requires = \
['certbot>=1.23.0,<2.0.0',
 'dnspod-sdk>=0.0.2,<0.0.3',
 'zope.interface>=5.4.0,<6.0.0']

entry_points = \
{'certbot.plugins': ['certbot-dnspod = certbot_dnspod:Authenticator']}

setup_kwargs = {
    'name': 'certbot-dnspod',
    'version': '0.1.0',
    'description': 'A certbot plugin for DNSPod',
    'long_description': '# certbot-dnspod\n\nA certbot plugin for DNSPod\n\n## 安装\n\n```\npip install certbot-dnspod\n```\n\n## 创建证书\n\n```\nsudo certbot certonly \\\n--authenticator certbot-dnspod \\\n--certbot-dnspod-credentials ~/.secrets/certbot/dnspod.ini \\\n-d example.com \\\n-d *.myfunpinpin.com\n```\n\n其中~/.secrets/certbot/dnspod.ini为配置文件路径，内容\n\n```\ncertbot_dnspod_token_id = <your token id>\ncertbot_dnspod_token = <your token>\n```\n\nchmod\n\n```\nchmod 600 ~/.secrets/certbot/dnspod.ini\n```\n\n## 参数\n\n官方插件是参数形式是\n```\n--dns-cloudflare-credentials\n```\n\n而第三方插件的参数是::\n\n```\n--authenticator certbot-dnspod\n```\n\n或者\n\n```\n-a certbot-dnspod\n```\n\n## 其他\n\n- [certbot命令行参数](https://eff-certbot.readthedocs.io/en/stable/using.html#certbot-command-line-options)\n- [编写一个certbot插件](https://certbot.eff.org/docs/contributing.html#writing-your-own-plugin)\n- [官方插件](https://certbot.eff.org/docs/using.html#dns-plugins)\n- [三方插件](https://certbot.eff.org/docs/using.html#third-party-plugins)\n- [poetry加自定义的entry_points](https://python-poetry.org/docs/pyproject/#plugins)\n\n',
    'author': 'codeif',
    'author_email': 'me@codeif.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/codeif/certbot-dnspod',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
